/**
 ** On-Screen-Display for Noatun
 ** Copyright (c) 2001, Flaming Sword Productions
 ** All Rights Reserved under the BSD License
 **
 ** This plug-in REQUIRES that you have the X On-Screen-Display (xosd)
 ** libraries installed.
 **
 ** $Id: kosd.cpp,v 1.10 2002/05/31 15:29:56 brian Exp $
 **/
#include	<kdebug.h>
#include	<kwin.h>
#include	<klocale.h>
#include	<qapplication.h>
#include	<qpainter.h>
#include	<qbitmap.h>
#include	<qimage.h>
#include	<qpixmap.h>
#include	<qfont.h>
#include	<qwidget.h>
#include	<qobject.h>
#include	<qtimer.h>
#include	<qlabel.h>
#include	"kosd.h"

/**
 ** KOSD::KOSD(): CONSTRUCTOR. 
 **/
		KOSD::KOSD()
		:QLabel(0L, "KOSD", WStyle_Customize|WStyle_StaysOnTop|WStyle_NoBorderEx ),
		_position (Bottom), 
		_style (Normal), 
		_status (Stopped), 
		_font (QFont("helvetica",18,QFont::Bold)),
		_color (Qt::green), 
		_volume(-1),
		_quieterVolume(true),
		_hideTimeout(3),
		_displayTimer (0) {

	KWin::setState( winId(), NET::Modal | NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager );
	KWin::setOnAllDesktops( winId(), true );

	emit setTextFont( _font );
	emit setTextColor( _color );

	setText( QString::null );
//	setBackgroundOrigin( WindowOrigin );
	setBackgroundMode( NoBackground );
	setAlignment( AlignBottom );
	setAutoResize( true );

	_setPosition();
}

/**
 ** KOSD::~KOSD(): DESTRUCTOR. 
 **/
		KOSD::~KOSD() {
}

/**
 ** KOSD::_setPosition(): Update our position.
 **/
void		KOSD::setPosition( Position p ) {

	_position = p;
	_setPosition();
}

/**
 ** KOSD::setStyle(): Update our style.
 **/
void		KOSD::setStyle( Style s ) {

	_style = s;
}

/**
 ** KOSD::setStatus(): Set our current operational status.
 **/
void		KOSD::setStatus( Status s ) {

	_status = s;
	if( _displayTimer ) {
		_displayTimer->stop();
		delete _displayTimer;
		_displayTimer = 0L;
		_messageStack.clear();
	}
	showText(_lastMessage);
}

/**
 ** KOSD::showVolume(): We're configured to show the volume
 ** quietly.
 **/
void		KOSD::showVolume() {

	if( _displayTimer ) {
		_displayTimer->stop();
		delete _displayTimer;
		_displayTimer = 0L;
		_messageStack.clear();
	}

	QString show = "        "+_volumeString()+"\n"; // trailing newline is a _good_ thing.
	_push( show );
	setText( show );
	emit _showLabel();
}

/**
 ** KOSD::setTextColor(): Set the text color.
 **/
void		KOSD::setTextColor( const QColor& c ) {

	_color = c;

	QPalette p(Qt::black);
	p.setColor( QColorGroup::Foreground, _color );
	setPalette( p );
}

/**
 ** KOSD::setFont(): Set the text color.
 **/
void		KOSD::setTextFont( const QFont& c ) {

	_font = c;
	setFont( _font );
}

/**
 ** KOSD::setVolume(): Someone adjusted our volume.
 **/
void		KOSD::setVolume( int volume ) {

	_volume = volume;
	if( _displayTimer ) {
		_displayTimer->stop();
		delete _displayTimer;
		_displayTimer = 0L;
		_messageStack.clear();
	}
	if( _quieterVolume ) {
		emit showVolume();
	} else {
		emit showText(_lastMessage);
	}
}

/**
 ** KOSD::showText(): Emit this signal when you want KOSD to show
 ** something.
 **/
void		KOSD::showText( const QString& s ) {

	QString show = _statusString()+" "+_volumeString()+"\n"+s;

	_lastMessage = s;
	_push( show );

	if( !_displayTimer ) {
		setText( show );
		emit _showLabel();
	}
}

/**
 ** KOSD::_setPosition(): Based on the screen geometry and configured parameters,
 ** move the label to a point on the screen.
 **
 ** NOTICE: This is not going to support Xinerama initially.  Given enough public
 ** support, it may, eventually.
 **/
void		KOSD::_setPosition() {

	move( 10, 
		( (_position==Bottom) ?
			QApplication::desktop()->height() - height() - 50
			:50 
		) 
	);

/*	
	QLabel f(this);
	f.setFont( _font );
	int height = f.sizeHint().height()*3;
	QRect screenGeometry = QApplication::desktop()->geometry();

	QRect myGeometry = QRect(
		10,
		(_position==Bottom?screenGeometry.height()-height:10),
		screenGeometry.width(),
		height
	);

	setGeometry( myGeometry ); */
}

/**
 ** KOSD::_showLabel(): Emit this to show the label.  It will set
 ** a hide() timer automagically.
 **/
void		KOSD::_showLabel() {


	/**
	 ** If there is no displaytimer set, we need to initialize
	 ** things.
	 **/
	if(!_displayTimer) {
		_displayTimer = new QTimer( this );
		connect( _displayTimer, SIGNAL(timeout()), this, SLOT(_showLabel()));
		_displayTimer->start( 1000 * _hideTimeout, false );
			// Is this right??  3 second * 1000 = 3000 milliseconds??

		show();
	} else {
		QString s = _pop();

		if(s.isNull()) {
			hide();
			delete _displayTimer;
			_displayTimer = 0L;
		} else {
			setText( s );
		}
	}
}

/**
 ** KOSD::_statusString(): Return a QString representing our current
 ** status.
 **/
QString		KOSD::_statusString() {

	QString status;
	switch( _status ) {
		case(Playing):	status = ">> Play ";	break;
		case(Stopped):	status = "[] Stop ";	break;
		case(Paused):	status = "|| Pause";	break;
	}
	return( status );
}

/**
 ** KOSD::_volumeString(): Return a QString representing our current
 ** volume level.  The volume should be set by a signal connected to
 ** KOSD::setVolume().
 **/
QString		KOSD::_volumeString() {

	if( _volume < 0 ) return( QString("") );

	QString volume = "";
	for( int i = 0; i < (_volume/2); i++ ) volume.append("|");
	for( int i = (_volume/2)+1; i <= 50; i++ ) volume.append("-");

	return( i18n("Volume: ") + volume );
}

/**
 ** KOSD::_push(): Push a string into our queue.  We'll be removing
 ** strings from the end of the list at a timely interval for display.
 **/
void		KOSD::_push( const QString& s ) {

	_messageStack.enqueue( new QString(s) );
}

/**
 ** KOSD::_pop(): Pop a string from our queue.
 **/
QString		KOSD::_pop() {

	if(_messageStack.isEmpty()) return( QString::null );

	QString s = (_messageStack.current())->copy();
	_messageStack.remove();

	return( s );
}

/**
 ** KOSD::paintEvent(): Congratulations.  You are the lucky winner of the
 ** Least-Wanted-To-Override contest... Ugh.  :)
 **/
void		KOSD::paintEvent( QPaintEvent *qpe ) {
QPainter	p( this );
//QBrush		b( Qt::gray, Qt::SolidPattern );
QBitmap		mask( size() );

	mask.fill( color0 );
 
        p.setFont( _font );
	p.setBackgroundMode( TransparentMode );

	switch( _style ) {

		/**
		 ** Shadowed Text - but not alpha-rendered, dang it... :)
		 **/
		case( Shadowed ):
			p.setPen( Qt::black );
			p.setBrush( Qt::black );
			p.drawText( rect().x()+1, rect().y()+1, rect().width(), rect().height(),
				AlignBottom, text() );
			p.setPen( _color );
			p.setBrush( _color );
			p.drawText( rect(), AlignBottom, text() );
//			mask = pixmap();
//			setMask( mask );
		break;

		/**
		 ** This is our DEFAULT look - which is really just
		 ** the same thing as calling QLabel::paintEvent()...
		 ** Boy howdy, I _love_ C++... ;)
		 **/
		default:
			QLabel::paintEvent( qpe );
		break;
	}
}

/**
 ** KOSD::resizeEvent(): Because we want the QLabel to automagically resize itself
 ** to fit whatever's in there, we ought to move it to the appropriate location in
 ** the event that it _does_ change its size (after configuration, for example).
 **/
void		KOSD::resizeEvent( QResizeEvent *qre ) {

	kdDebug() << "Resize to " << qre->size().width() << "x" << qre->size().height() << endl;

	// Start by passing the resizeEvent() to our parent.
	QLabel::resizeEvent( qre );

	// Then readjust our position on the screen.
	_setPosition();

}
