/**
 ** On-Screen-Display for Noatun
 ** Copyright (c) 2001, Flaming Sword Productions
 ** All Rights Reserved under the BSD License
 **
 ** This plug-in REQUIRES that you have the X On-Screen-Display (xosd)
 ** libraries installed.
 **
 ** (ok, actually it doesn't.  XOSD doesn't do what I want, so I rewrote
 ** it.  Rat finks...)
 **
 ** $Id: osd.cpp,v 1.12 2002/05/31 15:29:56 brian Exp $
 **/
#include	<klocale.h>
#include	<kconfig.h>
#include	<noatun/app.h>
#include	<noatun/player.h>
#include	"kosd.h"
#include	"osd.h"
#include	"osdconfig.h"

/**
 ** OSD::OSD(): CONSTRUCTOR. 
 **/
		OSD::OSD()
		:QObject(0, "FSP-OSD-module"), Plugin() {

	NOATUNPLUGINC(OSD);

	/**
	 ** Load configuration
	 **/
	const QFont *defaultf = new QFont("fixed", 20, QFont::Bold);
	const QColor *defaultc = new QColor(Qt::red);

	KGlobal::config()->setGroup("Noatun OSD");
        QFont font = KGlobal::config()->readFontEntry("Font", defaultf );
        QColor color = KGlobal::config()->readColorEntry("TextColor", defaultc );
	int textpos = KGlobal::config()->readNumEntry("TextPosition", 0 );
	int style = KGlobal::config()->readNumEntry("TextStyle", 0 );
	bool quieterVol = KGlobal::config()->readBoolEntry("QuieterVolume", false );
	_showFilename = KGlobal::config()->readBoolEntry("FilenameOnly", false );
	int hideTimeout = KGlobal::config()->readNumEntry("HideTimeout", 1 );

	if( (hideTimeout > 10) || (hideTimeout < 1) ) hideTimeout = 1;

	/**
	 ** This is a MUCH better way of doing things than libxosd, no?  ;)
	 **/
	_osd = new KOSD();
	_osd->setTextFont( font );
	_osd->setTextColor( color );
	_osd->setPosition((KOSD::Position)textpos);
	_osd->setStyle((KOSD::Style)style);
	_osd->setQuieterVolume(quieterVol);
	_osd->setHideTimeout(hideTimeout);

	connect( napp->player(), SIGNAL(newSong()), this, SLOT(showSong()));
	connect( napp->player(), SIGNAL(volumeChanged(int)), this, SLOT(setVolume(int)));
	connect( napp->player(), SIGNAL(playing()), this, SLOT(showSong()));
	connect( napp->player(), SIGNAL(playing()), this, SLOT(showPlay()));
	connect( napp->player(), SIGNAL(paused()), this, SLOT(showPause()));
//sure would be nice if there were a way to know when my configuration
//has been changed...  will hafta check the docs to see if there is.. :)

	OSDConfig *_config = new OSDConfig( this );
	connect(_config, SIGNAL(saved()), this, SLOT(loadSettings()));

	emit _osd->showText( i18n("Welcome to KDE Media Player!") );
}

/**
 ** OSD::~OSD(): DESTRUCTOR. 
 **/
		OSD::~OSD() {

//	if( _osd ) xosd_uninit( _osd );
	if( _osd ) delete( _osd );
}

/**
 ** OSD::loadSettings(): Load our settings.
 **/
void		OSD::loadSettings() {

	const QFont *defaultf = new QFont("fixed", 20, QFont::Bold);
	const QColor *defaultc = new QColor(Qt::red);

	KGlobal::config()->setGroup("Noatun OSD");
	QFont font = KGlobal::config()->readFontEntry("Font", defaultf );
	QColor color = KGlobal::config()->readColorEntry("TextColor", defaultc );
	int textpos = KGlobal::config()->readNumEntry("TextPosition", 0 );
	int style = KGlobal::config()->readNumEntry("TextStyle", 0 );
	bool quieterVol = KGlobal::config()->readBoolEntry("QuieterVolume", false );
	_showFilename = KGlobal::config()->readBoolEntry("FilenameOnly", false );
	int hideTimeout = KGlobal::config()->readNumEntry("HideTimeout", 1 );

	if( (hideTimeout > 10) || (hideTimeout < 1) ) hideTimeout = 1;

	_osd->setTextFont( font );
	_osd->setTextColor( color );
	_osd->setPosition((KOSD::Position)textpos);
	_osd->setStyle((KOSD::Style)style);
	_osd->setQuieterVolume(quieterVol);
	_osd->setHideTimeout(hideTimeout);

	emit _osd->showText("(Configuration changed.)");
}

/**
 ** OSD::showSong(): Use the libosd libraries to show which song this is.
 ** (I'll need to cre8 a prefs dialog before too long...)
 **/
void		OSD::showSong() {

	if(!_osd) return;
	if (!napp->player()->current()) return;

//	xosd_display( _osd, 1, XOSD_printf, "%s", item->file().ascii() );

	if( _showFilename ) 
		emit _osd->showText( napp->player()->current().file() );
	else
		emit _osd->showText( napp->player()->current().title() );
}

/**
 ** OSD::showPlay(): Show that we are playing.
 **/
void		OSD::showPlay() {

	if(!_osd) return;
	emit _osd->setStatus( KOSD::Playing );
}

/**
 ** OSD::showPause(): Show that we are paused.
 **/
void		OSD::showPause() {

	if(!_osd) return;
	emit _osd->setStatus( KOSD::Paused );
}

/**
 ** OSD::showStop(): Show that we are stopped.
 **/
void		OSD::showStop() {

	if(!_osd) return;
	emit _osd->setStatus( KOSD::Stopped );
}


/**
 ** OSD::setVolume(): The volume was adjusted.
 **/
void            OSD::setVolume( int v ) {

        if(!_osd) return;
        emit _osd->setVolume( v );
}

/**-------------------------------------------
 ** Module loader interface (boy, I love KDE!)
 **/
extern "C" {

Plugin		*create_plugin() {
	return new OSD();
}

};
